-- ==========================================================
--
--              CHAPTER 1
--
-- ==========================================================


-- 2

SELECT      CountryName
           ,COUNT(SD.StockID) AS CarsSold
           ,(SELECT COUNT(SalesDetailsID)
             FROM Data.SalesDetails) AS SalesTotal
FROM       Data.SalesDetails SD 
INNER JOIN Data.Sales AS SA ON SA.SalesID = SD.SalesID
INNER JOIN Data.Customer CU ON SA.CustomerID = CU.CustomerID
INNER JOIN Data.Country CO ON CU.Country = CO.CountryISO2
GROUP BY   CountryName	


-- 3

SELECT     MakeName
           ,SUM(SalePrice) AS SalePrice
           ,SUM(SalePrice) / (SELECT SUM(SalePrice) 
                              FROM Data.SalesDetails) AS SalesRatio
FROM       Data.Make AS MK
INNER JOIN Data.Model AS MD ON MK.MakeID = MD.MakeID
INNER JOIN Data.Stock AS ST ON ST.ModelID = MD.ModelID
INNER JOIN Data.SalesDetails SD ON ST.StockCode = SD.StockID
GROUP BY   MakeName	


-- 4

SELECT     ST.Color
FROM       Data.Stock AS ST 	
INNER JOIN Data.SalesDetails SD ON ST.StockCode = SD.StockID
WHERE      SD.SalePrice = (SELECT MAX(SalePrice) FROM Data.SalesDetails)	


-- 5

SELECT     MK.MakeName, MD.ModelName, RepairsCost
FROM       Data.Make AS MK	
INNER JOIN Data.Model AS MD ON MK.MakeID = MD.MakeID
INNER JOIN Data.Stock AS ST ON ST.ModelID = MD.ModelID	
WHERE      RepairsCost > 3 * (SELECT AVG(RepairsCost) FROM Data.Stock)


-- 6

SELECT     MK.MakeName, MD.ModelName, Cost, RepairsCost
FROM       Data.Make AS MK
INNER JOIN Data.Model AS MD ON MK.MakeID = MD.MakeID
INNER JOIN Data.Stock AS ST ON ST.ModelID = MD.ModelID
WHERE      RepairsCost BETWEEN	
                        (SELECT AVG(RepairsCost) FROM Data.Stock) * 0.9
		           AND	
                        (SELECT AVG(RepairsCost) FROM Data.Stock) * 1.1


-- 7

SELECT     MakeName, AVG(SD.SalePrice) AS AverageUpperSalePrice
FROM       Data.Make AS MK
INNER JOIN Data.Model AS MD ON MK.MakeID = MD.MakeID
INNER JOIN Data.Stock AS ST ON ST.ModelID = MD.ModelID
INNER JOIN Data.SalesDetails SD ON ST.StockCode = SD.StockID
GROUP BY   MakeName		
HAVING     AVG(SD.SalePrice) > 2 * (SELECT AVG(SalePrice)
                                    FROM   Data.SalesDetails)


-- 8

SELECT     MakeName, SalePrice
FROM       Data.Make AS MKX 
INNER JOIN Data.Model AS MDX ON MKX.MakeID = MDX.MakeID
INNER JOIN Data.Stock AS STX ON STX.ModelID = MDX.ModelID
INNER JOIN Data.SalesDetails SDX ON STX.StockCode = SDX.StockID
WHERE      MakeName IN (	
                        SELECT     TOP (5) MK.MakeName
                        FROM       Data.Make AS MK 
                        INNER JOIN Data.Model AS MD 
                                   ON MK.MakeID = MD.MakeID
                        INNER JOIN Data.Stock AS ST 
                                   ON ST.ModelID = MD.ModelID
                        INNER JOIN Data.SalesDetails SD 
                                   ON ST.StockCode = SD.StockID
                        INNER JOIN Data.Sales SA
                                   ON SA.SalesID = SD.SalesID
                        GROUP BY   MK.MakeName
                        ORDER BY   SUM(SA.TotalSalePrice) DESC
                        )
ORDER BY   MakeName, SalePrice DESC


-- 9

SELECT      MK.MakeName
           ,COUNT(MK.MakeName) AS VehiclesSold
           ,SUM(SD.SalePrice) AS TotalSalesPerMake
FROM       Data.Make AS MK
INNER JOIN Data.Model AS MD ON MK.MakeID = MD.MakeID
INNER JOIN Data.Stock AS ST ON ST.ModelID = MD.ModelID
INNER JOIN Data.SalesDetails SD ON ST.StockCode = SD.StockID
WHERE      MakeName IN (
                        SELECT     TOP (3) MK.MakeName
                        FROM       Data.Make AS MK 
                        INNER JOIN Data.Model AS MD 
                                   ON MK.MakeID = MD.MakeID
                        INNER JOIN Data.Stock AS ST 
                                   ON ST.ModelID = MD.ModelID
                        INNER JOIN Data.SalesDetails SD 
                                   ON ST.StockCode = SD.StockID
                        INNER JOIN Data.Sales AS SA 
                                   ON SA.SalesID = SD.SalesID
                        GROUP BY    MK.MakeName
                        ORDER BY   COUNT(MK.MakeName) DESC
                       )
GROUP BY  MK.MakeName
ORDER BY  VehiclesSold DESC


-- 10

SELECT      DISTINCT Color
FROM        Data.Stock STX
INNER JOIN  Data.SalesDetails SDX 
            ON STX.StockCode = SDX.StockID
WHERE       SalesID IN (	
                        SELECT     TOP 5 PERCENT SalesID
                        FROM       Data.Stock AS ST
                        INNER JOIN Data.SalesDetails SD 
                                   ON ST.StockCode = SD.StockID
                        ORDER BY   (SD.SalePrice - 
                                           (Cost + ISNULL(RepairsCost, 0)
                                            + PartsCost + TransportInCost)
                                    ) ASC
			      )


-- 11

SELECT     TOP 5 MK.MakeName, MD.ModelName, SD.SalePrice 
FROM       Data.Make AS MK
INNER JOIN Data.Model AS MD ON MK.MakeID = MD.MakeID
INNER JOIN Data.Stock AS ST ON ST.ModelID = MD.ModelID
INNER JOIN Data.SalesDetails SD ON ST.StockCode = SD.StockID
INNER JOIN Data.Sales AS SA ON SA.SalesID = SD.SalesID 
WHERE      Color IN (SELECT     ST.Color
                     FROM       Data.Model AS MD
                     INNER JOIN Data.Stock AS ST 
                                ON ST.ModelID = MD.ModelID
                     INNER JOIN Data.SalesDetails SD 
                                ON ST.StockCode = SD.StockID
                     WHERE      SD.SalePrice = 
                                      (
                                       SELECT      MAX(SD.SalePrice) 
                                       FROM        Data.SalesDetails SD
                                       INNER JOIN  Data.Sales SA
                                       ON SA.SalesID = SD.SalesID 
		                           )
                     )
ORDER BY   SD.SalePrice DESC


-- 12

SELECT     MakeName, SUM(SA.TotalSalePrice) AS TotalSales
FROM       Data.Make AS MK INNER JOIN Data.Model AS MD 
           ON MK.MakeID = MD.MakeID
INNER JOIN Data.Stock AS ST ON ST.ModelID = MD.ModelID
INNER JOIN Data.SalesDetails SD ON ST.StockCode = SD.StockID
INNER JOIN Data.Sales AS SA ON SA.SalesID = SD.SalesID
INNER JOIN Data.Customer CU ON SA.CustomerID = CU.CustomerID
INNER JOIN Data.Country CO ON CU.Country = CO.CountryISO2
WHERE      CountryName NOT IN (
                               SELECT     TOP 4 CO.CountryName
                               FROM       Data.Sales AS SA 
                               INNER JOIN Data.Customer CU 
                                          ON SA.CustomerID = CU.CustomerID
                               INNER JOIN Data.Country CO 
                                          ON CU.Country = CO.CountryISO2
                               GROUP BY   CountryName
                               ORDER BY   SUM(SA.TotalSalePrice) ASC
                               )
GROUP BY   MakeName


-- 13

SELECT     MakeName
           ,SUM(SD.SalePrice) AS SalePrice
           ,SUM(SD.SalePrice) / 
                        (SELECT     SUM(SD.SalePrice) 
		             FROM       Data.SalesDetails SD 
                         INNER JOIN Data.Sales AS SA
                                    ON SA.SalesID = SD.SalesID
                         WHERE      YEAR(SaleDate) = 2015
                        ) AS SalesRatio
FROM       Data.Make AS MK
INNER JOIN Data.Model AS MD ON MK.MakeID = MD.MakeID
INNER JOIN Data.Stock AS ST ON ST.ModelID = MD.ModelID
INNER JOIN Data.SalesDetails SD ON ST.StockCode = SD.StockID
INNER JOIN Data.Sales AS SA ON SA.SalesID = SD.SalesID
WHERE      YEAR(SaleDate) = 2015
GROUP BY   MakeName


-- 14

SELECT      MK.MakeName
           ,MD.ModelName
           ,SD.SalePrice AS ThisYearsSalePrice
           ,SD.SalePrice
             - (SELECT  AVG(SD.SalePrice) 
		    FROM       Data.Stock ST
                INNER JOIN Data.SalesDetails SD 
                           ON ST.StockCode = SD.StockID
                INNER JOIN Data.Sales AS SA 
                           ON SA.SalesID = SD.SalesID
                WHERE      YEAR(SaleDate) = 2015) AS DeltaToLastYearAverage
FROM       Data.Make AS MK INNER JOIN Data.Model AS MD 
           ON MK.MakeID = MD.MakeID
INNER JOIN Data.Stock AS ST ON ST.ModelID = MD.ModelID
INNER JOIN Data.SalesDetails SD ON ST.StockCode = SD.StockID
INNER JOIN Data.Sales AS SA ON SA.SalesID = SD.SalesID
WHERE      YEAR(SA.SaleDate) = 2016

